#!/bin/bash

if [ ${#} != 3 ]; then
	echo "ERROR: 3 parameters expected, ${#} found."
	echo "Usage: ${0} <tasksim_config_file> <output_trace> <trace>"
	if [ "`uname -m`" = "riscv64" ] ; then
		echo "Example ${0} /gpfs/scratch/bsc18/bsc18292/romol/musa_riscv_redistributable/tasksim/etc/conf/riscv_reference.conf ./output_trace /apps/paraver_sample_traces/fft/RISCV_fft.1024"
	else
		echo "Example ${0} /gpfs/scratch/bsc18/bsc18292/romol/musa_riscv_redistributable/tasksim/etc/conf/riscv_reference.conf ./output_trace /apps/TASKSIM/input_samples/apxy/axpy_4096_in128"
	fi
	exit 1;
elif [ ! -f "${1}" ]; then
	echo "ERROR: tasksim configuration file ${1} not found."
	exit 1;
# elif [ ! -d `echo "${2}" | rev | cut -d '/' -f1 | rev` ]; then
#	 echo "ERROR: Output directory does not exist. You need to create it."
#	 exit 1;
elif [ ! -f "${3}.prv" ]; then
	echo "ERROR: ${3}.prv file is missing from the trace files."
	exit 1;
elif [ ! -f "${3}.pcf" ]; then
	echo "ERROR: ${3}.pcf file is missing from the trace files."
	exit 1;
elif [ ! -f "${3}.row" ]; then
	echo "ERROR: ${3}.row file is missing from the trace files."
	exit 1;
fi



if [ -f "${2}".pcf ] || [ -f "${2}".prv ] || [ -f "${2}".row ] || [ -f "${2}".stats ]; then
	echo "WARNING: Output files ${2} will be overwritten."
fi

if [ ! -f `grep "latency_file" ${1} | tr -d ' ' | cut -d '=' -f2` ]; then
	echo "WARNING: latency file is missing."
	latency_file=""
else
	latency_file=`grep "latency_file" ${1} | tr -d ' ' | cut -d '=' -f2`
fi

if [ -z `grep "inorder_commit" ${1} | tr -d ' ' | cut -d '=' -f2` ]; then
	echo "WARNING: inorder_commit flag not specified in the config file."
	inorder_commit=0
else
	inorder_commit=`grep "inorder_commit" ${1} | tr -d ' ' | cut -d '=' -f2`
fi

if [ -z `grep "register_file_length" ${1} | tr -d ' ' | cut -d '=' -f2` ]; then
	echo "WARNING: register_file_length not specified in the config file."
	vector_unit_throughput=512
else
	vector_unit_throughput=`grep "register_file_length" ${1} | tr -d ' ' | cut -d '=' -f2`
fi

config_file=${1}
result=${2}
trace=${3}

ts=`echo ${0} | rev | cut --complement -d '/' -f1 | rev`
binary=riscvsim

echo "Checking configuration file parameters..."
${ts}/check_config_file -c ${config_file} -x ${ts}/../etc/conf/parameter_checklist.txt
if [ $? -ne 0 ]; then
    echo "Configuration file had some errors, correct them before running the simulation."
    exit
else
    echo "Configuration file has all necessary parameters."
fi
start=`date +%s`

echo "Simulating ${ts}/${binary} ${config_file} ${trace} -c MemCPU=RISCV -c MemCPU.prv_trace=${result} -c prv_trace=${result} -c memory_model=PERFECT -c private_cache_levels=1 -c shared_cache_levels=0 &> ${result}.stats"
${ts}/${binary} ${config_file} ${trace} -c MemCPU=RISCV -c MemCPU.prv_trace=${result} -c prv_trace=${result} -c memory_model=PERFECT -c private_cache_levels=1 -c shared_cache_levels=0 &> ${result}.stats
if [ $? -ne 0 ]; then
    end=`date +%s`
    secs=$(( (end - start) ))
    time=`date -d@${secs} -u +%H:%M:%S`
    echo "Simulation ended after ${time} with error code $?, printing simulator output ${result}.stats"
    cat ${result}.stats
else
    end=`date +%s`
    secs=$(( (end - start) ))
    time=`date -d@${secs} -u +%H:%M:%S`
    echo "Simulation done. It took ${time}."
    # CPU
    freq_mhz=$(grep "cpu_frequency" ${config_file} | sed 's/[^0-9]//g')
    duration=$(grep "Final cycle count:" ${result}.stats | sed 's/[^0-9]//g')
    instructions=$(grep ":ROA total number of committed instructions:ROA:0" ${result}.stats | grep -o "^[0-9]*")
    stall_total=$(grep ":ROA total number of commit stalled cycles:ROA:0" ${result}.stats | grep -o "^[0-9]*")
    stall_load=$(grep ":ROA total number of commit stalled cycles due to a load instruction:ROA:0" ${result}.stats | grep -o "^[0-9]*")
    stall_store=$(grep ":ROA total number of commit stalled cycles due to a store instruction:ROA:0" ${result}.stats | grep -o "^[0-9]*")
    stall_nonmem=$(grep ":ROA total number of commit stalled cycles due to a non-memory instruction:ROA:0" ${result}.stats | grep -o "^[0-9]*")
    ipc=$(echo "scale=2;$instructions/$duration" | bc)
    if [ $stall_total -ne 0 ]; then
        stall_load_ratio=$(echo "scale=2;100.0*$stall_load/$stall_total" | bc)
        stall_store_ratio=$(echo "scale=2;100.0*$stall_store/$stall_total" | bc)
        stall_nonmem_ratio=$(echo "scale=2;100.0*$stall_nonmem/$stall_total" | bc)
    else
        stall_load_ratio=0
        stall_store_ratio=0
        stall_nonmem_ratio=0
    fi
    # CPU-Cache
    cpu_stores=$(grep ":ROA number of stores sent:" ${result}.stats | sed 's/:.*//g')
    cpu_nt_stores=$(grep ":ROA number of non-temporal stores sent:" ${result}.stats | sed 's/:.*//g')
    cpu_loads=$(grep ":ROA number of loads sent:" ${result}.stats | sed 's/:.*//g')
    cpu_nt_loads=$(grep ":ROA number of non-temporal loads sent:" ${result}.stats | sed 's/:.*//g')
    #echo "CPU freq = $freq_mhz, duration = $duration, $cpu_stores, $cpu_loads, $cpu_nt_stores, $cpu_nt_loads"
    cpu_requests=$(echo "$cpu_loads+$cpu_stores+$cpu_nt_loads+$cpu_nt_stores" | bc)
    cpu_requests_cycle=$(echo "scale=3; $cpu_requests/$duration" | bc)
    cpu_bw=$(echo "scale=3; 64*$cpu_requests/$duration*($freq_mhz/1000)" | bc )
    # Cache
    l1_mshr_entries=$(grep "\[DL1MSHR\]" -A1 ${config_file} | grep "size" | sed 's/[^0-9]//g')
    stall_cache_total=$(grep ":stalled cycles:CacheL1:0" ${result}.stats | grep -o "^[0-9]*")
    stall_cache_mshr=$(grep ":stalled cycles by a full MSHR:CacheL1:0" ${result}.stats | grep -o "^[0-9]*")
    stall_cache_cacheset=$(grep ":stalled cycles by a full CacheSet:CacheL1:0" ${result}.stats | grep -o "^[0-9]*")
    stall_cache_victim=$(grep ":stalled cycles by full victim cache:CacheL1:0" ${result}.stats | grep -o "^[0-9]*")
    stall_cache_coherence=$(grep ":stalled cycles to mantain coherence:CacheL1:0" ${result}.stats | grep -o "^[0-9]*")
    l1_requests=$(grep ":requests-processed:CacheL1:0" ${result}.stats | grep -o "^[0-9]*")
    r_miss=$(grep ":read_miss:CacheL1:0" ${result}.stats | grep -o "^[0-9]*")
    w_miss=$(grep ":write_miss:CacheL1:0" ${result}.stats | grep -o "^[0-9]*")
    r_half_miss=$(grep ":read_half_miss:CacheL1:0" ${result}.stats | grep -o "^[0-9]*")
    w_half_miss=$(grep ":write_half_miss:CacheL1:0" ${result}.stats | grep -o "^[0-9]*")
    cache_miss_ratio=$(echo "scale=2;100.0*($r_miss+$w_miss+$r_half_miss+$w_half_miss)/$l1_requests" | bc)
    if [ $stall_cache_total -ne 0 ]; then
        stall_mshr_ratio=$(echo "scale=2;100.0*$stall_cache_mshr/$stall_cache_total" | bc)
        stall_cacheset_ratio=$(echo "scale=2;100.0*$stall_cache_cacheset/$stall_cache_total" | bc)
        stall_victim_ratio=$(echo "scale=2;100.0*$stall_cache_victim/$stall_cache_total" | bc)
        stall_coherence_ratio=$(echo "scale=2;100.0*$stall_cache_coherence/$stall_cache_total" | bc)
    else
        stall_mshr_ratio=0
        stall_cachheset_ratio=0
        stall_victim_ratio=0
        stall_coherence_ratio=0
    fi
    # Cache-DRAM
    cache_requests=$(grep ":requests-sent:CacheL1:" ${result}.stats | sed 's/:.*//g')
    cache_requests_cycle=$(echo "scale=3; $cache_requests/$duration" | bc)
    cache_bw=$(echo "scale=3; 64*$cache_requests/$duration*($freq_mhz/1000)" | bc )
    # DRAM:
    config_dram_bw_value=$(grep "bandwidth" ${config_file} | sed 's/[^0-9]//g')
    config_dram_bw=$(echo "scale=3;$config_dram_bw_value/1000000000.0" | bc)
    dram_latency=$(grep "\[Memory\]" -A5 ${config_file} | grep "latency" | sed 's/[^0-9]//g')

    mshr_bw=$(echo "scale=3; 64*$l1_mshr_entries/$dram_latency" | bc)
    echo "Average IPC: $ipc"
    echo "CPU stalled $stall_total cycles, ${stall_load_ratio}% due to a loads, ${stall_store_ratio}% due to stores, ${stall_nonmem_ratio}% due to non-memory instructions."
    echo "Average CPU-L1Cache requests sent per cycle: $cpu_requests_cycle, bw: $cpu_bw GB/s"
    echo "Cache miss ratio is ${cache_miss_ratio}%"
    echo "Cache stalled $stall_cache_total cycles, ${stall_mshr_ratio}% due to a full MSHR, ${stall_cacheset_ratio}% due to a CacheSet full of pending requests, ${stall_victim_ratio}% due to a full Writeback Buffer, ${stall_coherence_ratio}% to mantain coherence."
    echo "Average L1Cache-DRAM requests sent per cycle: $cache_requests_cycle, bw: $cache_bw GB/s"
    echo "Configuration DRAM bandwidth: $config_dram_bw GB/s, and MSHR limits it to $mshr_bw GB/s read bandwidth"
fi

dpt=$(grep "disable_paraver_trace" ${config_file})
dpt=${dpt%%#*}
dpt=${dpt##*=}
if [[ ! -z $dpt ]]; then
    if [[ $dpt -eq 0 ]]; then
        cp ${trace}.row ${result}.row
    fi
else
    cp ${trace}.row ${result}.row
fi
